// various dialogs for SuperWebMailer/SuperMailingList
// copyright 2023 mirko boeer

const supported_targetgroups_Elements = ["P", "DIV", "A", "SPAN", "IMG", "HR", "H1", "H2", "H3", "H4", "H5", "H6", "TABLE", "TR"];

tinymce.PluginManager.add('externaldialogs', (editor, url) => {

  function getAscendant(sel, supported_targetgroups_Elements){
    if( supported_targetgroups_Elements.includes(sel.tagName) )
      return sel;
    var element = sel.parentElement;

      while (element && element.tagName != "BODY"){
        if(supported_targetgroups_Elements.includes(element.tagName))
          return element;
        element = element.parentElement;
      }

    return null;
  }

  let language = editor.options.get("language");

  var FunctionsTitle = language == "de" ? 'Funktionen' : 'Functions';
  var FunctionsEdit = language == "de" ? 'Funktionen bearbeiten...' : 'Edit functions...';

  var TextblocksTitle = language == "de" ? HTMLEntityDecode('Textbl&ouml;cke') : 'Text blocks';
  var TextblocksEdit = language == "de" ? HTMLEntityDecode('Textbl&ouml;cke bearbeiten...') : 'Edit text blocks...';

  var LoadFileTitle = language == "de" ? HTMLEntityDecode('HTML-Datei laden...') : 'Load HTML file...';
  var TargetGroupsTitle = language == "de" ? HTMLEntityDecode('Zielgruppen') : 'Target groups';

  var notAllowedTags = ["IMG", "TABLE", "TR", "HR", "OBJECT", "EMBED"];

  var functionsDialogAction = function(){
        var date = new Date();
        var nocache = date.getTime() / 1000;

        var w = editor.windowManager.openUrl({
          title: FunctionsTitle,
          url: editor.config.functionsDialogPath + "?nocache=" + nocache + "&tinymce=6&form=tinymce&formElement=tinymce&ckCsrfToken=" + tinymce.getCsrfToken(),
          width: 720,
          height: 400
        });
  }

  editor.ui.registry.addIcon('functions', DownloadFile(tinymce.defaulteditorConfig.baseMyPluginsUrl + 'externaldialogs/functions.svg'));

  editor.ui.registry.addMenuButton('functionsDialogButton', {
    //text: FunctionsTitle,
    icon: "functions",
    tooltip: FunctionsTitle,
    onAction: () => {
        functionsDialogAction();
    },
    onSetup: aMenuButton=>{
        const isEditable = ()=>{
            var sel = editor.selection.getStart();
            aMenuButton.setEnabled( editor.selection.isEditable() && !notAllowedTags.includes(sel.tagName)  )
        };
        return editor.on("NodeChange", isEditable),
        isEditable(),
        ()=>{
            editor.off("NodeChange", isEditable)
        }
    },
    fetch: function (callback) {
        callback(GetFunctionsMenuItems());
    }


  });

  editor.ui.registry.addNestedMenuItem('mnuFunctionsDialog', {
    text: FunctionsTitle,
    icon: "functions",
    onAction: () => {
      functionsDialogAction();
    },
    onSetup: aMenuButton=>{
        const isEditable = ()=>{
            var sel = editor.selection.getStart();
            aMenuButton.setEnabled( editor.selection.isEditable() && !notAllowedTags.includes(sel.tagName)  )
        };
        return editor.on("NodeChange", isEditable),
        isEditable(),
        ()=>{
            editor.off("NodeChange", isEditable)
        }
    },
    getSubmenuItems: () => GetFunctionsMenuItems()
  });


  function GetFunctionsMenuItems(){


      var date = new Date();
      var nocache = date.getTime() / 1000;

      var url = editor.config.functionsDialogPath + '?AsJSON&IsFCKEditor=true&tinymce=6&ckCsrfToken=' + tinymce.getCsrfToken() + '&nocache=' + nocache;

      let _Items = [];
      let j = DownloadFile(url).trim();
      if(j.length > 1 && j[0] == "[" && j[j.length - 1] == "]")
         _Items = JSON.parse(j);

      _Items.unshift(FunctionsEdit);

      var items = [];
      for(var i=0; i<_Items.length; i++){

        const value = pluginHTMLEntityDecode( _Items[i] );
        var a = function () {
              editor.insertContent(value);
            }

        var item =
          {
            type: 'menuitem',
            text: pluginHTMLEntityDecode( _Items[i] ),
            icon: i == 0 ? "edit-block" : "functions",
            tooltip: value,
            onAction: i == 0 ? functionsDialogAction : a
          };
        items.push(item);
      }
      return items;
   }

/////////////

  let SML = false;
  if(typeof IsSML !== "undefined" && IsSML)
   SML = true;

  editor.ui.registry.addIcon('textblocks', DownloadFile(tinymce.defaulteditorConfig.baseMyPluginsUrl + 'externaldialogs/textblocks.svg'));

  if(!SML){

    var textblocksDialogAction = function(){
          var date = new Date();
          var nocache = date.getTime() / 1000;

          var w = editor.windowManager.openUrl({
            title: TextblocksTitle,
            url: editor.config.textblocksDialogPath + "?nocache=" + nocache + "&tinymce=6&form=tinymce&formElement=tinymce&ckCsrfToken=" + tinymce.getCsrfToken(),
            width: 720,
            height: 400
          });
    }

    editor.ui.registry.addMenuButton('textblocksDialogButton', {
     // text: TextblocksTitle,
      tooltip: TextblocksTitle,
      icon: "textblocks",
      onAction: () => {
          textblocksDialogAction();
      },
      onSetup: aMenuButton=>{
          const isEditable = ()=>{
              var sel = editor.selection.getStart();
              aMenuButton.setEnabled( editor.selection.isEditable() && !notAllowedTags.includes(sel.tagName)  )
          };
          return editor.on("NodeChange", isEditable),
          isEditable(),
          ()=>{
              editor.off("NodeChange", isEditable)
          }
      },
      fetch: function (callback) {
          callback(GetTextblocksMenuItems());
      }
    });

    editor.ui.registry.addNestedMenuItem('mnuTextblocksDialog', {
      text: TextblocksTitle,
      icon: "textblocks",
      onAction: () => {
        textblocksDialogAction();
      },
      onSetup: aMenuButton=>{
          const isEditable = ()=>{
              var sel = editor.selection.getStart();
              aMenuButton.setEnabled( editor.selection.isEditable() && !notAllowedTags.includes(sel.tagName)  )
          };
          return editor.on("NodeChange", isEditable),
          isEditable(),
          ()=>{
              editor.off("NodeChange", isEditable)
          }
      },
      getSubmenuItems: () => GetTextblocksMenuItems()
    });

    function GetTextblocksMenuItems(){


        var date = new Date();
        var nocache = date.getTime() / 1000;

        var url = editor.config.textblocksDialogPath + '?AsJSON&IsFCKEditor=true&tinymce=6&ckCsrfToken=' + tinymce.getCsrfToken() + '&nocache=' + nocache;

        let _Items = [];
        let j = DownloadFile(url).trim();
        if(j.length > 1 && j[0] == "[" && j[j.length - 1] == "]")
           _Items = JSON.parse(j);

        _Items.unshift(TextblocksEdit);

        var items = [];
        for(var i=0; i<_Items.length; i++){

          const value = pluginHTMLEntityDecode( _Items[i] );
          var a = function () {
                editor.insertContent(value);
              }

          var item =
            {
              type: 'menuitem',
              text: pluginHTMLEntityDecode( _Items[i] ),
              icon: i == 0 ? "edit-block" : "textblocks",
              tooltip: value,
              onAction: i == 0 ? textblocksDialogAction : a
            };
          items.push(item);
        }
        return items;
     }
  }
/////////////

  editor.ui.registry.addIcon('loadhtmlfile', DownloadFile(tinymce.defaulteditorConfig.baseMyPluginsUrl + 'externaldialogs/loadhtml.svg'));

  var loadfileDialogAction = function(){
        var date = new Date();
        var nocache = date.getTime() / 1000;

        var w = editor.windowManager.openUrl({
          title: LoadFileTitle,
          url: editor.config.loadfileDialogPath + "?nocache=" + nocache + "&tinymce=6&form=tinymce&formElement=tinymce&ckCsrfToken=" + tinymce.getCsrfToken(),
          width: 720,
          height: 600
        });
  }

  editor.ui.registry.addButton('loadfileDialogButton', {
    //text: LoadFileTitle,
    icon: "loadhtmlfile",
    tooltip: LoadFileTitle,
    onAction: () => {
        loadfileDialogAction();
    }
  });

  editor.ui.registry.addMenuItem('mnuLoadfileDialog', {
    text: LoadFileTitle,
    icon: "loadhtmlfile",
    onAction: () => {
      loadfileDialogAction();
    }
  });


/////////////

  editor.ui.registry.addIcon('targetgroups', DownloadFile(tinymce.defaulteditorConfig.baseMyPluginsUrl + 'externaldialogs/targetgroup.svg'));

  const targetGroupssupported = !SML && document.getElementById("TargetGroupsSupport") && document.getElementById("TargetGroupsSupport").value == "1" && document.getElementById("TargetGroupsDefined") && document.getElementById("TargetGroupsDefined").value == "1";

  if( targetGroupssupported ){


    editor.ui.registry.addMenuButton('targetGroupsButton', {
      //text: TargetGroupsTitle,
      tooltip: TargetGroupsTitle,
      icon: "targetgroups",

      onSetup: aMenuButton=>{

            const isEditable = ()=>{


                if(!targetGroupssupported){
                  aMenuButton.setEnabled(false);
                  return;
                }

                var sel = editor.selection.getStart();
                aMenuButton.setEnabled( editor.selection.isEditable() && getAscendant(sel, supported_targetgroups_Elements)  );
            };

          return editor.on("NodeChange", isEditable),
          isEditable(),
          ()=>{
              editor.off("NodeChange", isEditable)
          }
      },

      fetch: function (callback) {
        if(targetGroupssupported)
          callback(GetMenuItems());
          else
          callback([]);
      }

    });

    editor.ui.registry.addNestedMenuItem('mnuTargetgroups', {
      text: TargetGroupsTitle,
      icon: "targetgroups",
      onSetup: aMenuButton=>{

            const isEditable = ()=>{

                if(!targetGroupssupported){
                  aMenuButton.setEnabled(false);
                  return;
                }

                var sel = editor.selection.getStart();
                aMenuButton.setEnabled( editor.selection.isEditable() && getAscendant(sel, supported_targetgroups_Elements)  );
            };

          return editor.on("NodeChange", isEditable),
          isEditable(),
          ()=>{
              editor.off("NodeChange", isEditable)
          }
      },
      getSubmenuItems: () => targetGroupssupported ? GetMenuItems() : []
    });

    editor.ui.registry.addContextMenu('targetgroups', {
      update: (element) => editor.selection.isEditable() && getAscendant(editor.selection.getStart(), supported_targetgroups_Elements) ? 'mnuTargetgroups' : ''
    });

    function GetMenuItems(){

        var items = [];

        var date = new Date();
        var nocache = date.getTime() / 1000;

        var url = editor.config.loadtargetGroups + '?IsFCKEditor=false&tinymce=6&ckCsrfToken=' + tinymce.getCsrfToken() + '&nocache=' + nocache;

        let TargetGroupsItems = [];
        let j = DownloadFile(url).trim();
        if(j.length > 1 && j[0] == "[" && j[j.length - 1] == "]")
           TargetGroupsItems = JSON.parse(j);

//TargetGroupsItems.push("G1");
//TargetGroupsItems.push("G2");

        var sel = editor.selection.getStart();
        var element = getAscendant(sel, supported_targetgroups_Elements);
        var t = element && element.getAttribute('target_groups') && element.getAttribute('target_groups') ? element.getAttribute('target_groups') : "";

        for(var i=0; i<TargetGroupsItems.length; i++){

          const targetgroup = TargetGroupsItems[i];
          const tageName = "<" + element.tagName + ">";

          let x = t.split(" ");
          var checked = x.contains(targetgroup);

          var a = function () {
                var sel = editor.selection.getStart();
                element = getAscendant(sel, supported_targetgroups_Elements);
                var t = element.getAttribute('target_groups');
                if(!t)
                 t = targetgroup;
                 else{
                   let x = t.split(" ");
                   var exists = x.contains(targetgroup);

                   if(!exists)
                     t += " " + targetgroup;
                     else{
                       t = "";
                       for(var i=0; i<x.length; i++)
                        if(x[i] !== targetgroup)
                           t += " " + x[i];
                     }
                 }
                element.setAttribute('target_groups', t.trim());
              }

          var item =
            {
              type: 'togglemenuitem',
              text: tageName + " -> " + TargetGroupsItems[i],
              icon: "targetgroups",
              tooltip: targetgroup,
              onAction: a,
              selected: checked,
              onSetup: (api) => {
                return () => {};
              }
            };
          if(checked)
           item.active = true;
          items.push(item);
        }

        return items;
    }

  }

  return {
    name: 'ExternalDialogsPlugin',
    url: 'https://www.superwebmailer.de/'
  };

});



/////////////


